/** 
 *   Camera Control for STAR I.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdarg.h>
#include <unistd.h>
#include <malloc.h>
#include "camera.h"
#include "camera_header.h"
#include "my_stdio.h"


#define SWAP_BYTE(b) b = (((b << 8) & 0xff00) | ((b >> 8) & 0x00ff))

/******************************************************************************
 * Camera controller commands:
 *
 **/

int get_exposure_time(int dev, int *time)
{
	return get_int_from_cc(dev, "A", time);
}
		

int set_exposure_time(int dev, int time)
{
	return put_int_to_cc(dev, "A", time);
}


int get_gain(int dev, int *gain)
{
	return get_int_from_cc(dev, "B", gain);
}

		
int set_gain(int dev, int gain)
{
	if (gain == GAIN_HIGH || gain == GAIN_LOW)
		return put_int_to_cc(dev, "B", gain);
	else
		return -1;
}


int get_temp_regulation(int dev, int *temp)
{
	return get_int_from_cc(dev, "C", temp);
}

		
int set_temp_regulation(int dev, int temp)
{
	if (temp == TEMP_REG_ON || temp == TEMP_REG_OFF)
		return put_int_to_cc(dev, "C", temp);
	else
		return -1;
}


int get_box_y0(int dev, int *y)
{
	return get_int_from_cc(dev, "D", y);
}
		

int set_box_y0(int dev, int y)
{
	return put_int_to_cc(dev, "D", y);
}


int get_box_x0(int dev, int *x)
{
	return get_int_from_cc(dev, "E", x);
}
		

int set_box_x0(int dev, int x)
{
	return put_int_to_cc(dev, "E", x);
}


int get_box_y_len(int dev, int *y)
{
	return get_int_from_cc(dev, "F", y);
}
		

int set_box_y_len(int dev, int y)
{
	return put_int_to_cc(dev, "F", y);
}


int get_box_x_len(int dev, int *x)
{
	return get_int_from_cc(dev, "G", x);
}
		

int set_box_x_len(int dev, int x)
{
	return put_int_to_cc(dev, "G", x);
}


int get_zoom_factor(int dev, int *z)
{
	return get_int_from_cc(dev, "H", z);
}
		

int set_zoom_factor(int dev, int z)
{
	return put_int_to_cc(dev, "H", z);
}


int get_auto_win(int dev, int *w)
{
	return get_int_from_cc(dev, "I", w);
}

		
/* This function turns the automatic window calculation function off
   and on */
int set_auto_win(int dev, int w)
{
	return put_int_to_cc(dev, "I", w);
}


int get_date_time(int dev, time_t *time)
{
	PVERB(PWARN,"get_date_time: not implemented\n");
	return -1; /* FIXME */
}
		

int set_date_time(int dev, time_t time)
{
	struct tm *t;
	char date_str[8];

	t = localtime(&time);
	sprintf(date_str,"%02d%02d%02d", 
		t->tm_mon + 1, t->tm_mday, t->tm_year + 1900 - 2000); 
	if (put_str_to_cc(dev, "L", date_str) != 0)
		return -1;

	sprintf(date_str,"%02d%02d%02d", t->tm_hour, t->tm_min, t->tm_sec);
	if (put_str_to_cc(dev, "M", date_str) != 0)
		return -1;

	return 0;
}


int get_win_max(int dev, int *w)
{
	return get_int_from_cc(dev, "N", w);
}


int set_win_max(int dev, int w)
{
	return put_int_to_cc(dev, "N", w);
}


int get_win_min(int dev, int *w)
{
	return get_int_from_cc(dev, "O", w);
}


int set_win_min(int dev, int w)
{
	return put_int_to_cc(dev, "O", w);
}


static int cc_header_swap_byte(struct cc_image_head *h)
{
	SWAP_BYTE(h->exposure);
	SWAP_BYTE(h->x0);
	SWAP_BYTE(h->y0);
	SWAP_BYTE(h->x_len);
	SWAP_BYTE(h->y_len);
	SWAP_BYTE(h->temperature);
	SWAP_BYTE(h->gain);

	return 0;
}


int get_cc_header(int dev, struct cc_image_head *h)
{
	int ret;
	ret = get_str_from_cc(dev, "P", (char *) h, 
			      sizeof(struct cc_image_head));
	if (ret == 0) {
		cc_header_swap_byte(h);
	}

	return ret;
}


int set_cc_header(int dev, struct cc_image_head h)
{
	/* cc_header_swap_byte(&h); */
	PVERB(PWARN,"set_cc_header: not implemented\n");
	return -1; /* FIXME */
}


int get_binning(int dev, int *b)
{
	return get_int_from_cc(dev, "Q", b);
}


int set_binning(int dev, int b)
{
	return put_int_to_cc(dev, "Q", b);
}


int get_roi_index(int dev, int *i)
{
	return get_int_from_cc(dev, "R", i);
}


int set_roi_index(int dev, int i)
{
	return put_int_to_cc(dev, "R", i);
}


int get_roi_delay(int dev, int *d)
{
	return get_int_from_cc(dev, "S", d);
}


int set_roi_delay(int dev, int d)
{
	return put_int_to_cc(dev, "S", d);
}


int get_temperature(int dev, int *t)
{
	struct cc_image_head h;
	
	if (get_cc_header(dev, &h) != 0) {
		return -1;
	}
	*t = (int) h.temperature;

	return 0;
}

/******************************************************************************
 * Camera controller action commands:
 *
 **/


int cc_act_exposure(int dev)
{
	return action_to_cc(dev, "A");
}


int cc_act_dark(int dev)
{
	return action_to_cc(dev, "B");
}


int cc_act_bias(int dev)
{
	return action_to_cc(dev, "C");
}


int cc_act_focus(int dev)
{
	return action_to_cc(dev, "D");
}


int cc_act_triger(int dev)
{
	return action_to_cc(dev, "E");
}


int cc_act_shade(int dev)
{
	return action_to_cc(dev, "G");
}


int cc_act_clear(int dev)
{
	return action_to_cc(dev, "H");
}


/* FIXME: cmd's: IJRSTUWX */


int cc_act_move_mouse(int dev, int x_delta, int y_delta)
{
	char s[12];

	sprintf(s, "V%d:%d", x_delta, y_delta);

	return action_to_cc(dev, s);
}


